import dash_cytoscape as cyto


class Graph:

    def __init__(self):
        self.nodes = []
        self.edges = []
        self.COUNT_THRESHOLD = 2
        self.MAX_NUM_WORDS = 10

    def get_all_words(self):
        all_words = [node_dict['data']['label'] for node_dict in self.nodes]
        return all_words

    def clear_graph_elements(self):
        self.nodes = []
        self.edges = []

    def fill_with_associations(self, word2vec_model, base_word):
        associated_words = word2vec_model.get_associated_words(base_word)
        self.clear_graph_elements()
        self.add_node(base_word, is_base_node=1)
        self.add_nodes(associated_words)
        self.add_edges(base_word, associated_words)

    def add_nodes(self, nodes):
        for node in nodes:
            self.add_node(node)

    def add_node(self, node, is_base_node=0):
        node_dict = {'data': {'id': node, 'label': node, 'is_base_node': is_base_node}}
        if is_base_node:
            node_dict['selectable'] = False
            node_dict['grabbable'] = False
        if node_dict not in self.nodes:
            self.nodes.append(node_dict)

    def add_edges(self, source, targets):
        for target in targets:
            self.add_edge(source, target)

    def add_edge(self, source, target):
        self.edges.append({'data': {'source': source, 'target': target}})

    def remove_node(self, node):
        node_idx_to_remove = -1
        for idx, node_dict in enumerate(self.nodes):
            if node_dict['data']['id'] == node:
                node_idx_to_remove = idx

        if node_idx_to_remove >= 0:
            self.nodes.pop(node_idx_to_remove)

        self.remove_edges(node)

    def remove_edges(self, node):
        edges_idxs_to_remove = []
        for idx, edge_dict in enumerate(self.edges):
            if edge_dict['data']['source'] == node or edge_dict['data']['target'] == node:
                edges_idxs_to_remove.append(idx)

        for edge_idx_to_remove in edges_idxs_to_remove:
            self.edges.pop(edge_idx_to_remove)

    def get_nodes_and_edges(self):
        return {'nodes': self.nodes, 'edges': self.edges}

    def set_nodes_and_edges(self, nodes_and_edges):
        self.nodes = nodes_and_edges['nodes']
        self.edges = nodes_and_edges['edges']

    def get_elements(self):
        elements = []
        elements.extend(self.nodes)
        elements.extend(self.edges)
        return elements

    def get_cytoscape_graph(self, component_id):
        elements = self.get_elements()
        return cyto.Cytoscape(id=component_id,
                              autoRefreshLayout=True,
                              layout={'name': 'cose', 'animate': True},
                              style={'width': '100%', 'height': '550px'},
                              elements=elements,
                              userZoomingEnabled=False,
                              userPanningEnabled=False,
                              stylesheet=[
                                  {
                                      'selector': '[is_base_node > 0.5]',
                                      'style': {
                                          'label': 'data(label)',
                                          'background-color': 'green'
                                      }
                                  },
                                  {
                                      'selector': '[is_base_node < 0.5]',
                                      'style': {
                                          'label': 'data(label)',
                                      }
                                  }
                              ]
                              )

    def extend_graph(self, word2vec_model, base_node):
        current_words = [node['data']['id'] for node in self.nodes]
        all_associated_words = []
        for current_word in current_words:
            associated_words = word2vec_model.get_associated_words(current_word, top_n=100)
            all_associated_words.extend(associated_words)

        associated_words_filtered = [word for word in all_associated_words if word not in current_words]
        associated_words_count = {word: associated_words_filtered.count(word) for word in list(set(associated_words_filtered))}

        common_associated_words = [word for word, count in associated_words_count.items() if count >= self.COUNT_THRESHOLD]
        while len(common_associated_words) > self.MAX_NUM_WORDS:
            self.COUNT_THRESHOLD += 1
            common_associated_words = [word for word, count in associated_words_count.items() if count >= self.COUNT_THRESHOLD]

        self.add_nodes(common_associated_words)
        self.add_edges(base_node, common_associated_words)
